@@ -13,15 +13,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from eventlet import greenthread
-from oslo_utils import uuidutils
-
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
-from neutron.common import exceptions
-from neutron import manager
-
 from networking_l2gw.db.l2gateway import db_query
 from networking_l2gw.db.l2gateway import l2gateway_models as models
 from networking_l2gw.extensions import l2gateway
@@ -31,7 +22,15 @@ from networking_l2gw.services.l2gateway.common import constants
 from networking_l2gw.services.l2gateway.common import l2gw_validators
 from networking_l2gw.services.l2gateway import exceptions as l2gw_exc
 
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
+from neutron_lib.db import api as db_api
+from neutron_lib.db import model_query
+from neutron_lib import exceptions
+from neutron_lib.plugins import directory
 from oslo_log import log as logging
+from oslo_utils import uuidutils
 from sqlalchemy.orm import exc as sa_orm_exc
 
 LOG = logging.getLogger(__name__)
@@ -45,20 +44,28 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
     connection_resource = constants.CONNECTION_RESOURCE_NAME
     config.register_l2gw_opts_helper()
 
+    @db_api.retry_if_session_inactive()
+    @db_api.CONTEXT_READER
     def _get_l2_gateway(self, context, gw_id):
         try:
-            gw = context.session.query(models.L2Gateway).get(gw_id)
+            gw = model_query.get_by_id(context, models.L2Gateway, gw_id)
         except sa_orm_exc.NoResultFound:
             raise l2gw_exc.L2GatewayNotFound(gateway_id=gw_id)
         return gw
 
+    @db_api.retry_if_session_inactive()
+    @db_api.CONTEXT_READER
     def _get_l2_gateways(self, context):
-        return context.session.query(models.L2Gateway).all()
+        return model_query.get_collection(context, models.L2Gateway,
+                                          dict_func=None)
 
+    @db_api.retry_if_session_inactive()
+    @db_api.CONTEXT_READER
     def _get_l2_gw_interfaces(self, context, id):
         return context.session.query(models.L2GatewayInterface).filter_by(
             device_id=id).all()
 
+    @db_api.CONTEXT_READER
     def _is_vlan_configured_on_any_interface_for_l2gw(self,
                                                       context,
                                                       l2gw_id):
@@ -74,6 +81,7 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
                     return True
         return False
 
+    @db_api.retry_if_session_inactive()
     def _get_l2_gateway_devices(self, context, l2gw_id):
         return context.session.query(models.L2GatewayDevice).filter_by(
             l2_gateway_id=l2gw_id).all()
@@ -133,7 +141,7 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
     def _retrieve_gateway_connections(self, context, gateway_id,
                                       mapping_info={}, only_one=False):
         filters = {'l2_gateway_id': [gateway_id]}
-        for k, v in mapping_info.iteritems():
+        for k, v in mapping_info.items():
             if v and k != constants.SEG_ID:
                 filters[k] = [v]
         query = self._get_collection_query(context,
@@ -147,78 +155,71 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
         gw = l2_gateway[self.gateway_resource]
         tenant_id = self._get_tenant_id_for_create(context, gw)
         devices = gw['devices']
-        self._validate_any_seg_id_empty_in_interface_dict(devices)
-        with context.session.begin(subtransactions=True):
-                gw_db = models.L2Gateway(
-                    id=gw.get('id', uuidutils.generate_uuid()),
-                    tenant_id=tenant_id,
-                    name=gw.get('name'))
-                context.session.add(gw_db)
-                l2gw_device_dict = {}
-                for device in devices:
-                    l2gw_device_dict['l2_gateway_id'] = id
-                    device_name = device['device_name']
-                    l2gw_device_dict['device_name'] = device_name
-                    l2gw_device_dict['id'] = uuidutils.generate_uuid()
-                    uuid = self._generate_uuid()
-                    dev_db = models.L2GatewayDevice(id=uuid,
-                                                    l2_gateway_id=gw_db.id,
-                                                    device_name=device_name)
-                    context.session.add(dev_db)
-                    for interface_list in device['interfaces']:
-                        int_name = interface_list.get('name')
-                        if constants.SEG_ID in interface_list:
-                            seg_id_list = interface_list.get(constants.SEG_ID)
-                            for seg_ids in seg_id_list:
-                                uuid = self._generate_uuid()
-                                interface_db = self._get_int_model(uuid,
-                                                                   int_name,
-                                                                   dev_db.id,
-                                                                   seg_ids)
-                                context.session.add(interface_db)
-                        else:
+        with db_api.CONTEXT_WRITER.using(context):
+            gw_db = models.L2Gateway(
+                id=gw.get('id', uuidutils.generate_uuid()),
+                tenant_id=tenant_id,
+                name=gw.get('name'))
+            context.session.add(gw_db)
+            l2gw_device_dict = {}
+            for device in devices:
+                l2gw_device_dict['l2_gateway_id'] = id
+                device_name = device['device_name']
+                l2gw_device_dict['device_name'] = device_name
+                l2gw_device_dict['id'] = uuidutils.generate_uuid()
+                uuid = self._generate_uuid()
+                dev_db = models.L2GatewayDevice(id=uuid,
+                                                l2_gateway_id=gw_db.id,
+                                                device_name=device_name)
+                context.session.add(dev_db)
+                for interface_list in device['interfaces']:
+                    int_name = interface_list.get('name')
+                    if constants.SEG_ID in interface_list:
+                        seg_id_list = interface_list.get(constants.SEG_ID)
+                        for seg_ids in seg_id_list:
                             uuid = self._generate_uuid()
                             interface_db = self._get_int_model(uuid,
                                                                int_name,
                                                                dev_db.id,
-                                                               0)
+                                                               seg_ids)
                             context.session.add(interface_db)
-                        context.session.query(models.L2GatewayDevice).all()
+                    else:
+                        uuid = self._generate_uuid()
+                        interface_db = self._get_int_model(uuid,
+                                                           int_name,
+                                                           dev_db.id,
+                                                           0)
+                        context.session.add(interface_db)
+                    context.session.query(models.L2GatewayDevice).all()
         return self._make_l2_gateway_dict(gw_db)
 
+    @db_api.CONTEXT_WRITER
     def update_l2_gateway(self, context, id, l2_gateway):
-        """Update l2 gateway."""
-        self._admin_check(context, 'UPDATE')
+        """Update L2Gateway."""
         gw = l2_gateway[self.gateway_resource]
-        if 'devices' in gw:
-            devices = gw['devices']
+        devices = gw.get('devices')
+        dev_db = None
+        l2gw_db = None
         with context.session.begin(subtransactions=True):
-                l2gw_db = self._get_l2_gateway(context, id)
-                if l2gw_db.network_connections:
-                    raise l2gw_exc.L2GatewayInUse(gateway_id=id)
-                dev_db = self._get_l2_gateway_devices(context, id)
-                if not gw.get('devices'):
-                    l2gw_db.name = gw.get('name')
-                    return self._make_l2_gateway_dict(l2gw_db)
+            l2gw_db = self._get_l2_gateway(context, id)
+            if not devices and l2gw_db:
+                l2gw_db.name = gw.get('name')
+                return self._make_l2_gateway_dict(l2gw_db)
+            if devices:
                 for device in devices:
                     dev_name = device['device_name']
-                    dev_db = self._get_l2gw_devices_by_name_andl2gwid(context,
-                                                                      dev_name,
-                                                                      id)
-                    if not dev_db:
-                        raise l2gw_exc.L2GatewayDeviceNotFound(device_id="")
+                    dev_db = (self._get_l2gw_devices_by_name_andl2gwid(
+                        context, dev_name, id))
+                    interface_dict_list = [i for i in device['interfaces']]
                     interface_db = self._get_l2_gw_interfaces(context,
                                                               dev_db[0].id)
                     self._delete_l2_gateway_interfaces(context, interface_db)
-                    interface_dict_list = []
-                    self.validate_device_name(context, dev_name, id)
-                    for interfaces in device['interfaces']:
-                        interface_dict_list.append(interfaces)
                     self._update_interfaces_db(context, interface_dict_list,
                                                dev_db)
-        if gw.get('name'):
-            l2gw_db.name = gw.get('name')
-        return self._make_l2_gateway_dict(l2gw_db)
+        if l2gw_db:
+            if gw.get('name'):
+                l2gw_db.name = gw.get('name')
+            return self._make_l2_gateway_dict(l2gw_db)
 
     def _update_interfaces_db(self, context, interface_dict_list, device_db):
         for interfaces in interface_dict_list:
@@ -240,27 +241,22 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
                                                    0)
                 context.session.add(interface_db)
 
+    @db_api.CONTEXT_READER
     def get_l2_gateway(self, context, id, fields=None):
         """get the l2 gateway by id."""
         self._admin_check(context, 'GET')
         gw_db = self._get_l2_gateway(context, id)
-        if gw_db:
-            return self._make_l2_gateway_dict(gw_db, fields)
-        else:
-            return []
+        return self._make_l2_gateway_dict(gw_db, fields)
 
     def delete_l2_gateway(self, context, id):
         """delete the l2 gateway  by id."""
-        self._admin_check(context, 'DELETE')
-        with context.session.begin(subtransactions=True):
-            gw_db = self._get_l2_gateway(context, id)
-            if gw_db is None:
-                raise l2gw_exc.L2GatewayNotFound(gateway_id=id)
-            if gw_db.network_connections:
-                raise l2gw_exc.L2GatewayInUse(gateway_id=id)
-            context.session.delete(gw_db)
-        LOG.debug("l2 gateway '%s' was deleted.", id)
+        gw_db = self._get_l2_gateway(context, id)
+        if gw_db:
+            with db_api.CONTEXT_WRITER.using(context):
+                context.session.delete(gw_db)
+            LOG.debug("l2 gateway '%s' was deleted.", id)
 
+    @db_api.CONTEXT_READER
     def get_l2_gateways(self, context, filters=None, fields=None,
                         sorts=None,
                         limit=None,
@@ -288,14 +284,13 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
 
     def _delete_l2_gateway_interfaces(self, context, int_db_list):
         """delete the l2 interfaces  by id."""
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             for interfaces in int_db_list:
                 context.session.delete(interfaces)
         LOG.debug("l2 gateway interfaces was deleted.")
 
     def create_l2_gateway_connection(self, context, l2_gateway_connection):
         """Create l2 gateway connection."""
-        self._admin_check(context, 'CREATE')
         gw_connection = l2_gateway_connection[self.connection_resource]
         l2_gw_id = gw_connection.get('l2_gateway_id')
         network_id = gw_connection.get('network_id')
@@ -306,32 +301,35 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
         if constants.SEG_ID in gw_connection:
             segmentation_id = gw_connection.get(constants.SEG_ID)
             nw_map[constants.SEG_ID] = segmentation_id
-        is_vlan = self._is_vlan_configured_on_any_interface_for_l2gw(context,
-                                                                     l2_gw_id)
-        network_id = l2gw_validators.validate_network_mapping_list(nw_map,
-                                                                   is_vlan)
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             gw_db = self._get_l2_gateway(context, l2_gw_id)
             tenant_id = self._get_tenant_id_for_create(context, gw_db)
-            if self._retrieve_gateway_connections(context,
-                                                  l2_gw_id,
-                                                  nw_map):
-                raise l2gw_exc.L2GatewayConnectionExists(mapping=nw_map,
-                                                         gateway_id=l2_gw_id)
             nw_map['tenant_id'] = tenant_id
-            connection_id = uuidutils.generate_uuid()
-            nw_map['id'] = connection_id
             if not segmentation_id:
                 nw_map['segmentation_id'] = "0"
-            gw_db.network_connections.append(
-                models.L2GatewayConnection(**nw_map))
-            gw_db = models.L2GatewayConnection(id=connection_id,
-                                               tenant_id=tenant_id,
-                                               network_id=network_id,
-                                               l2_gateway_id=l2_gw_id,
-                                               segmentation_id=segmentation_id)
-        return self._make_l2gw_connections_dict(gw_db)
+            conn_db = self._retrieve_gateway_connections(context, l2_gw_id,
+                                                         nw_map)
+            if not conn_db:
+                connection_id = uuidutils.generate_uuid()
+                nw_map['id'] = connection_id
+                gw_db.network_connections.append(
+                    models.L2GatewayConnection(**nw_map))
+                gw_db = models.L2GatewayConnection(
+                    id=connection_id,
+                    tenant_id=tenant_id,
+                    network_id=network_id,
+                    l2_gateway_id=l2_gw_id,
+                    segmentation_id=segmentation_id)
+                return self._make_l2gw_connections_dict(gw_db)
+            return self._make_l2gw_connections_dict(conn_db[0])
 
+    @db_api.CONTEXT_READER
+    def get_l2_gateway_connections_count(self, context, filters=None):
+        return len(self._get_collection(context, models.L2GatewayConnection,
+                                        self._make_l2gw_connections_dict,
+                                        filters=filters))
+
+    @db_api.CONTEXT_READER
     def get_l2_gateway_connections(self, context, filters=None,
                                    fields=None,
                                    sorts=None, limit=None, marker=None,
@@ -347,6 +345,7 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
                                     marker_obj=marker_obj,
                                     page_reverse=page_reverse)
 
+    @db_api.CONTEXT_READER
     def get_l2_gateway_connection(self, context, id, fields=None):
         """Get l2 gateway connection."""
         self._admin_check(context, 'GET')
@@ -356,7 +355,6 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
 
     def delete_l2_gateway_connection(self, context, id):
         """Delete the l2 gateway connection by id."""
-        self._admin_check(context, 'DELETE')
         with context.session.begin(subtransactions=True):
             gw_db = self._get_l2_gateway_connection(context, id)
             context.session.delete(gw_db)
@@ -365,7 +363,7 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
     def _admin_check(self, context, action):
         """Admin role check helper."""
         # TODO(selva): his check should be required if the tenant_id is
-        # specified inthe request, otherwise the policy.json do a trick
+        # specified in the request, otherwise the policy.json do a trick
         # this need further revision.
         if not context.is_admin:
             reason = _('Cannot %s resource for non admin tenant') % action
@@ -448,6 +446,7 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
                 id="")
         return con
 
+    @db_api.CONTEXT_READER
     def _get_l2gw_ids_by_interface_switch(self, context, interface_name,
                                           switch_name):
         """Get l2 gateway ids by interface and switch."""
@@ -501,16 +500,104 @@ class L2GatewayMixin(l2gateway.L2GatewayPluginBase,
                 device_id=device_name)
         return gw
 
+    def validate_l2_gateway_for_create(self, context, l2_gateway):
+        self._admin_check(context, 'CREATE')
+        gw = l2_gateway[self.gateway_resource]
+        devices = gw['devices']
+        self._validate_any_seg_id_empty_in_interface_dict(devices)
+
+    def validate_l2_gateway_for_delete(self, context, l2gw_id):
+        self._admin_check(context, 'DELETE')
+        gw_db = self._get_l2_gateway(context, l2gw_id)
+        if gw_db.network_connections:
+            raise l2gw_exc.L2GatewayInUse(gateway_id=l2gw_id)
+        return
+
+    def validate_l2_gateway_for_update(self, context, id, l2_gateway):
+        self._admin_check(context, 'UPDATE')
+        gw = l2_gateway[self.gateway_resource]
+        devices = None
+        dev_db = None
+        if 'devices' in gw:
+            devices = gw['devices']
+        if not devices:
+            return
+        with db_api.CONTEXT_READER.using(context):
+            # Attemp to retrieve l2gw
+            gw_db = self._get_l2_gateway(context, id)
+            if devices:
+                for device in devices:
+                    dev_name = device['device_name']
+                    dev_db = self._get_l2gw_devices_by_name_andl2gwid(
+                        context, dev_name, id)
+                    if not dev_db:
+                        raise l2gw_exc.L2GatewayDeviceNotFound(
+                            device_id=dev_name)
+                    self.validate_device_name(context, dev_name, id)
+                    interface_list = device['interfaces']
+                    if not interface_list:
+                        raise l2gw_exc.L2GatewayInterfaceRequired()
+                    if constants.SEG_ID in interface_list[0]:
+                        for interfaces in interface_list:
+                            if constants.SEG_ID not in interfaces:
+                                raise l2gw_exc.L2GatewaySegmentationRequired()
+                    if constants.SEG_ID not in interface_list[0]:
+                        for interfaces in interface_list[1:]:
+                            if constants.SEG_ID in interfaces:
+                                raise l2gw_exc.L2GatewaySegmentationRequired()
+                    if gw_db.devices:
+                        if gw_db.devices[0].interfaces[0]['segmentation_id']:
+                            if constants.SEG_ID not in interface_list[0]:
+                                raise l2gw_exc.L2GatewaySegmentationIDExists()
+                        else:
+                            if constants.SEG_ID in interface_list[0]:
+                                raise l2gw_exc.\
+                                    L2GatewaySegmentationIDNotExists()
+
+    def validate_l2_gateway_connection_for_create(self, context,
+                                                  l2_gateway_connection):
+        self._admin_check(context, 'CREATE')
+        gw_connection = l2_gateway_connection[self.connection_resource]
+        l2_gw_id = gw_connection.get('l2_gateway_id')
+        network_id = gw_connection.get('network_id')
+        plugin = directory.get_plugin()
+        plugin.get_network(context, network_id)
+        nw_map = {}
+        nw_map['network_id'] = network_id
+        nw_map['l2_gateway_id'] = l2_gw_id
+        segmentation_id = ""
+        if constants.SEG_ID in gw_connection:
+            segmentation_id = gw_connection.get(constants.SEG_ID)
+            nw_map[constants.SEG_ID] = segmentation_id
+        is_vlan = self._is_vlan_configured_on_any_interface_for_l2gw(context,
+                                                                     l2_gw_id)
+        network_id = l2gw_validators.validate_network_mapping_list(nw_map,
+                                                                   is_vlan)
+        with db_api.CONTEXT_WRITER.using(context):
+            if self._retrieve_gateway_connections(context,
+                                                  l2_gw_id,
+                                                  nw_map):
+                raise l2gw_exc.L2GatewayConnectionExists(mapping=nw_map,
+                                                         gateway_id=l2_gw_id)
+
+    @db_api.CONTEXT_READER
+    def validate_l2_gateway_connection_for_delete(self, context,
+                                                  l2_gateway_conn_id):
+        self._admin_check(context, 'DELETE')
+        gw_db = self._get_l2_gateway_connection(context,
+                                                l2_gateway_conn_id)
+        if gw_db is None:
+            raise l2gw_exc.L2GatewayConnectionNotFound(
+                id=l2_gateway_conn_id)
+
 
 def l2gw_callback(resource, event, trigger, **kwargs):
-    l2gwservice = manager.NeutronManager.get_service_plugins().get(
-        constants.L2GW)
+    l2gwservice = directory.get_plugin(constants.L2GW)
     context = kwargs.get('context')
     port_dict = kwargs.get('port')
     if l2gwservice:
         if event == events.AFTER_UPDATE:
-            greenthread.spawn_n(l2gwservice.add_port_mac,
-                                context, port_dict)
+            l2gwservice.add_port_mac(context, port_dict)
         elif event == events.AFTER_DELETE:
             l2gwservice.delete_port_mac(context, port_dict)
 
