@@ -17,6 +17,8 @@ from oslo_utils import timeutils
 from sqlalchemy import asc
 from sqlalchemy.orm import exc
 
+from neutron_lib.db import api as db_api
+
 from networking_l2gw.db.l2gateway.ovsdb import models
 
 LOG = logging.getLogger(__name__)
@@ -490,8 +492,8 @@ def get_pending_ucast_mac_remote(context, ovsdb_identifier, mac,
 
 def get_all_pending_remote_macs_in_asc_order(context, ovsdb_identifier):
     """Get all the pending remote macs in ascending order of timestamp."""
-    session = context.session
-    with session.begin():
+    with db_api.CONTEXT_READER.using(context):
+        session = context.session
         return session.query(
             models.PendingUcastMacsRemote
             ).filter_by(ovsdb_identifier=ovsdb_identifier
@@ -501,8 +503,8 @@ def get_all_pending_remote_macs_in_asc_order(context, ovsdb_identifier):
 
 def get_all_ucast_mac_remote_by_ls(context, record_dict):
     """Get ucast macs remote that matches ls_id and ovsdb_identifier."""
-    session = context.session
-    with session.begin():
+    with db_api.CONTEXT_READER.using(context):
+        session = context.session
         return session.query(models.UcastMacsRemotes).filter_by(
             ovsdb_identifier=record_dict['ovsdb_identifier'],
             logical_switch_id=record_dict['logical_switch_id']).all()
