@@ -16,9 +16,7 @@
 import random
 
 import eventlet
-
-from neutron.i18n import _LE
-
+from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
@@ -31,15 +29,21 @@ from networking_l2gw.services.l2gateway import exceptions
 LOG = logging.getLogger(__name__)
 
 
+class Activity(object):
+    Initial, Update = range(2)
+
+
 class OVSDBMonitor(base_connection.BaseConnection):
     """Monitors OVSDB servers."""
-    def __init__(self, conf, gw_config, callback):
-        super(OVSDBMonitor, self).__init__(conf, gw_config)
+    def __init__(self, conf, gw_config, callback, mgr=None):
+        super(OVSDBMonitor, self).__init__(conf, gw_config, mgr=None)
+        self.mgr = mgr
         self.rpc_callback = callback
         self.callbacks = {}
         self._setup_dispatch_table()
         self.read_on = True
         self.handlers = {"echo": self._default_echo_handler}
+        self.sock_timeout = cfg.CONF.ovsdb.socket_timeout
         if self.enable_manager:
             self.check_monitor_table_thread = False
         if not self.enable_manager:
@@ -93,37 +97,37 @@ class OVSDBMonitor(base_connection.BaseConnection):
     def set_monitor_response_handler(self, addr=None):
         """Monitor OVSDB tables to receive events for any changes in OVSDB."""
         if self.connected:
-                op_id = str(random.getrandbits(128))
-                props = {'select': {'initial': True,
-                                    'insert': True,
-                                    'delete': True,
-                                    'modify': True}}
-                monitor_message = {'id': op_id,
-                                   'method': 'monitor',
-                                   'params': [n_const.OVSDB_SCHEMA_NAME,
-                                              None,
-                                              {'Logical_Switch': [props],
-                                               'Physical_Switch': [props],
-                                               'Physical_Port': [props],
-                                               'Ucast_Macs_Local': [props],
-                                               'Ucast_Macs_Remote': [props],
-                                               'Physical_Locator': [props],
-                                               'Mcast_Macs_Local': [props],
-                                               'Physical_Locator_Set': [props]}
-                                              ]}
-                self._set_handler("update", self._update_event_handler)
-                if not self.send(monitor_message, addr=addr):
-                    # Return so that this will retried in the next iteration
-                    return
-                try:
-                    response_result = self._process_response(op_id)
-                except exceptions.OVSDBError:
-                    with excutils.save_and_reraise_exception():
-                        if self.enable_manager:
-                            self.check_monitor_table_thread = False
-                        LOG.exception(_LE("Exception while receiving the "
-                                          "response for the monitor message"))
-                self._process_monitor_msg(response_result, addr)
+            op_id = str(random.getrandbits(128))
+            props = {'select': {'initial': True,
+                                'insert': True,
+                                'delete': True,
+                                'modify': True}}
+            monitor_message = {'id': op_id,
+                               'method': 'monitor',
+                               'params': [n_const.OVSDB_SCHEMA_NAME,
+                                          None,
+                                          {'Logical_Switch': [props],
+                                           'Physical_Switch': [props],
+                                           'Physical_Port': [props],
+                                           'Ucast_Macs_Local': [props],
+                                           'Ucast_Macs_Remote': [props],
+                                           'Physical_Locator': [props],
+                                           'Mcast_Macs_Local': [props],
+                                           'Physical_Locator_Set': [props]}
+                                          ]}
+            self._set_handler("update", self._update_event_handler)
+            if not self.send(monitor_message, addr=addr):
+                # Return so that this will retried in the next iteration
+                return
+            try:
+                response_result = self._process_response(op_id)
+            except exceptions.OVSDBError:
+                with excutils.save_and_reraise_exception():
+                    if self.enable_manager:
+                        self.check_monitor_table_thread = False
+                    LOG.exception("Exception while receiving the "
+                                  "response for the monitor message")
+            self._process_monitor_msg(response_result, addr)
 
     def _update_event_handler(self, message, addr):
         self._process_update_event(message, addr)
@@ -136,7 +140,8 @@ class OVSDBMonitor(base_connection.BaseConnection):
             params_list = message.get('params')
             param_dict = params_list[1]
             self._process_tables(param_dict, data_dict)
-            self.rpc_callback(self._form_ovsdb_data(data_dict, addr))
+            self.rpc_callback(Activity.Update,
+                              self._form_ovsdb_data(data_dict, addr))
 
     def _process_tables(self, param_dict, data_dict):
         # Process all the tables one by one.
@@ -194,8 +199,8 @@ class OVSDBMonitor(base_connection.BaseConnection):
             else:
                 self.responses.append(json_m)
         except Exception as e:
-            LOG.exception(_LE("Exception [%s] while handling "
-                              "message"), e)
+            LOG.exception("Exception [%s] while handling "
+                          "message", e)
 
     def _rcv_thread(self):
         chunks = []
@@ -203,6 +208,16 @@ class OVSDBMonitor(base_connection.BaseConnection):
         prev_char = None
         while self.read_on:
             try:
+                # self.socket.recv() is a blocked call
+                # (if timeout value is not passed) due to which we cannot
+                # determine if the remote OVSDB server has died. The remote
+                # OVSDB server sends echo requests every 4 seconds.
+                # If there is no echo request on the socket for socket_timeout
+                # seconds(by default its 30 seconds),
+                # the agent can safely assume that the connection with the
+                # remote OVSDB server is lost. Better to retry by reopening
+                # the socket.
+                self.socket.settimeout(self.sock_timeout)
                 response = self.socket.recv(n_const.BUFFER_SIZE)
                 eventlet.greenthread.sleep(0)
                 if response:
@@ -216,8 +231,8 @@ class OVSDBMonitor(base_connection.BaseConnection):
                                                prev_char == '\\'):
                             rc += 1
                         if rc > lc:
-                            raise Exception(_LE("json string not valid"))
-                        elif lc == rc and lc is not 0:
+                            raise Exception("json string not valid")
+                        elif lc == rc and lc != 0:
                             chunks.append(response[message_mark:i + 1])
                             message = "".join(chunks)
                             eventlet.greenthread.spawn_n(
@@ -233,8 +248,8 @@ class OVSDBMonitor(base_connection.BaseConnection):
             except Exception as ex:
                 self.read_on = False
                 self.disconnect()
-                LOG.exception(_LE("Exception [%s] occurred while receiving"
-                                  "message from the OVSDB server"), ex)
+                LOG.exception("Exception [%s] occurred while receiving"
+                              "message from the OVSDB server", ex)
 
     def disconnect(self, addr=None):
         """disconnects the connection from the OVSDB server."""
@@ -247,9 +262,10 @@ class OVSDBMonitor(base_connection.BaseConnection):
         data_dict = self._initialize_data_dict()
         try:
             self._process_tables(result_dict, data_dict)
-            self.rpc_callback(self._form_ovsdb_data(data_dict, addr))
+            self.rpc_callback(Activity.Initial,
+                              self._form_ovsdb_data(data_dict, addr))
         except Exception as e:
-            LOG.exception(_LE("_process_monitor_msg:ERROR %s "), e)
+            LOG.exception("_process_monitor_msg:ERROR %s ", e)
 
     def _get_list(self, resource_list):
         return [element.__dict__ for element in resource_list]
