@@ -19,6 +19,7 @@ from oslo_utils import uuidutils
 
 from neutron.tests.unit import testlib_api
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 
 from networking_l2gw.db.l2gateway.ovsdb import lib
 from networking_l2gw.db.l2gateway.ovsdb import models
@@ -292,7 +293,7 @@ class OvsdbLibTestCase(testlib_api.SqlTestCase):
         if mac and logical_switch_uuid:
             record_dict['mac'] = mac
             record_dict['logical_switch_id'] = logical_switch_uuid
-        with self.ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             entry = models.UcastMacsRemotes(
                 uuid=record_dict['uuid'],
                 mac=record_dict['mac'],
@@ -305,10 +306,10 @@ class OvsdbLibTestCase(testlib_api.SqlTestCase):
 
     def test_get_ucast_mac_remote(self):
         record_dict = self._get_ucast_mac_remote_dict()
-        with self.ctx.session.begin(subtransactions=True):
-            entry = self._create_ucast_mac_remote(record_dict)
+        entry = self._create_ucast_mac_remote(record_dict)
         result = lib.get_ucast_mac_remote(self.ctx, record_dict)
-        self.assertEqual(entry, result)
+        for ent_key, ent_val in entry.items():
+            self.assertEqual(ent_val, result[ent_key])
 
     def test_add_ucast_mac_remote(self):
         record_dict = self._get_ucast_mac_remote_dict()
@@ -450,15 +451,15 @@ class OvsdbLibTestCase(testlib_api.SqlTestCase):
 
     def test_get_ucast_mac_remote_by_mac_and_ls(self):
         record_dict = self._get_ucast_mac_remote_dict()
-        with self.ctx.session.begin(subtransactions=True):
-            entry = self._create_ucast_mac_remote(record_dict,
-                                                  '00:11:22:33:44:55:66',
-                                                  'ls123')
+        entry = self._create_ucast_mac_remote(record_dict,
+                                              '00:11:22:33:44:55:66',
+                                              'ls123')
         record_dict['mac'] = '00:11:22:33:44:55:66'
         record_dict['logical_switch_uuid'] = 'ls123'
         result = lib.get_ucast_mac_remote_by_mac_and_ls(self.ctx,
                                                         record_dict)
-        self.assertEqual(entry, result)
+        for ent_key, ent_val in entry.items():
+            self.assertEqual(ent_val, result[ent_key])
 
     def test_get_ucast_mac_remote_by_mac_and_ls_when_not_found(self):
         record_dict = self._get_ucast_mac_remote_dict()
@@ -529,13 +530,15 @@ class OvsdbLibTestCase(testlib_api.SqlTestCase):
         record_dict1 = self._get_pending_mac_dict(timestamp1)
         timestamp2 = timeutils.utcnow()
         record_dict2 = self._get_pending_mac_dict(timestamp2)
-        with self.ctx.session.begin(subtransactions=True):
-            entry1 = self._create_pending_mac(record_dict1)
-            entry2 = self._create_pending_mac(record_dict2)
+        entry = []
+        with db_api.CONTEXT_WRITER.using(self.ctx):
+            entry.append(self._create_pending_mac(record_dict1))
+            entry.append(self._create_pending_mac(record_dict2))
         result = lib.get_all_pending_remote_macs_in_asc_order(
             self.ctx, record_dict1['ovsdb_identifier'])
-        self.assertEqual(result[0], entry1)
-        self.assertEqual(result[1], entry2)
+        for index, res in enumerate(result):
+            for k, v in res.items():
+                self.assertEqual(res[k], entry[index][k])
 
     def test_delete_pending_ucast_mac_remote(self):
         timestamp = timeutils.utcnow()
